@@ -1,4 +1,5 @@
 package Devel::DumpTrace;
+## no critic (NoStrict,StringyEval)
 
 use 5.008000;
 use Hash::SafeKeys;
@@ -10,10 +11,11 @@ use Devel::DumpTrace::CachedDisplayedHash;
 use IO::Handle;
 use File::Temp;
 use Carp;
+use Fcntl qw(:flock :seek);
 use strict;
 use warnings;
 
-our $VERSION = '0.25';
+our $VERSION = '0.26';
 
 my $Time_HiRes_avail;
 my $color_avail;
@@ -31,7 +33,15 @@ BEGIN {
     $Time_HiRes_avail = eval 'use Time::HiRes qw(time);1' || 0;
     $color_avail = eval
         'use Term::ANSIColor;$Term::ANSIColor::VERSION>=3.00' || 0;
-    
+
+    # idea from Devel::GlobalDestruction 0.13
+    # replace $_GLOBAL_DESTRUCTION used in earlier versions
+    if (defined ${^GLOBAL_PHASE}) {
+        eval 'sub __inGD(){${^GLOBAL_PHASE}eq q{DESTRUCT}&&__END()};1';
+    } else {
+        require B;
+        eval 'sub __inGD(){${B::main_cv()}==0&&__END();};1';
+    }
 }
 use Devel::DumpTrace::Const;
 
@@ -51,7 +61,6 @@ our %INCLUDE_PKG = ('main' => 1);
 our @EXCLUDE_PATTERN = ('^Devel::DumpTrace', '^Text::Shorten');
 our @INCLUDE_PATTERN = ();
 our (%DEFERRED, $PAD_MY, $PAD_OUR, $TRACE);
-our $_GLOBAL_DESTRUCTION = 0;
 our $_THREADS = 0;
 our $_INIT = 0;
 
@@ -70,7 +79,7 @@ my %esc = ("\a" => '\a', "\b" => '\b', "\t" => '\t', "\n" => '\n',
 # use PPI by default, if available
 $Devel::DumpTrace::NO_PPI
     || $ENV{DUMPTRACE_NOPPI}
-    || eval 'use Devel::DumpTrace::PPI;1';          ## no critic (StringyEval)
+    || eval 'use Devel::DumpTrace::PPI;1';
 
 {
     *Devel::Trace::TRACE = \$TRACE;
@@ -169,7 +178,7 @@ sub import {
 our $ZZ = 0;
 
 sub DB__DB {
-    return if $_GLOBAL_DESTRUCTION;
+    return if __inGD();
     return unless $Devel::DumpTrace::TRACE;
 
     my ($p, $f, $l) = caller();
@@ -193,7 +202,7 @@ sub DB__DB {
 
 sub get_source {
     my ($file, $line) = @_;
-    no strict 'refs';                    ## no critic (NoStrict)
+    no strict 'refs';
 
     if (!defined $sources{$file}) {
 	# die "source not available for $file ...\n";
@@ -349,7 +358,7 @@ sub current_depth {
 }
 
 sub refresh_pads {
-    return if $_GLOBAL_DESTRUCTION;
+    return if __inGD();
     my $current = current_depth();
     my $target = $PAD_MY->{__DEPTH__};
     if ($current >= $target) {
@@ -389,20 +398,8 @@ sub dumptrace {
         $out = join $/, map( $DUMPTRACE_COLOR . $_ . $DUMPTRACE_RESET,
                                split($/,$out)), "";
     }
-
-    if ($Devel::DumpTrace::LOCKOBJ) {
-        lock(my $lock = \$Devel::DumpTrace::LOCKOBJ);
-        print {$DUMPTRACE_FH} $out;
-    } elsif ($Devel::DumpTrace::LOCKFILE) {
-        use Fcntl qw(:flock :seek);
-        open my $lock, '>>', $Devel::DumpTrace::LOCKFILE;
-        flock $lock, LOCK_EX;
-        seek $DUMPTRACE_FH, 0, SEEK_END;
-        print {$DUMPTRACE_FH} $out;
-        close $lock;
-    } else {
-        print {$DUMPTRACE_FH} $out;
-    }
+    our $LOCKOBJ && lock(my $lock = \$LOCKOBJ);
+    print {$DUMPTRACE_FH} $out;
 }
 
 sub evaluate_and_display_line {
@@ -748,6 +745,7 @@ sub handle_ALL_deferred_output {
 sub handle_deferred_output {
     my ($sub, $file) = @_;
     my $deferred = pop @{$DEFERRED{"$sub : $file"}};
+    delete $DEFERRED{"$sub : $file"};
 
     if (defined $deferred) {
 
@@ -870,7 +868,7 @@ sub evaluate {
     my $v;
     _reset_dump();
 
-    no strict 'refs';                    ## no critic (NoStrict)
+    no strict 'refs';
 
     $deref_op ||= '';
     $index_op ||= '';
@@ -897,7 +895,7 @@ sub evaluate {
 	} elsif (defined $PAD_OUR->{__STALE__}{$sigvar}) {
 	    $v = $PAD_OUR->{__STALE__}{$sigvar};
 	} else {
-	    $v = eval "\\$pkgvar";                    ## no critic (StringyEval)
+	    $v = eval "\\$pkgvar";
 	}
 	if ($index_op eq '[') {
 	    return '[' . array_repr(${$v}, @keys) . ']->[';
@@ -926,7 +924,7 @@ sub evaluate {
 	} elsif (defined($PAD_OUR->{$sigvar})) {
 	    $v = $PAD_OUR->{$sigvar};
 	} else {
-	    $v = eval "\\$pkgvar";                    ## no critic (StringyEval)
+	    $v = eval "\\$pkgvar";
 	}
 	return '(' . hash_repr($v, @keys) . '){';
     }
@@ -950,7 +948,7 @@ sub evaluate {
 	    $v = $PAD_OUR->{$sigvar};
 	} else {
 	    eval {
-		$v = eval "\\" . $pkgvar;            ## no critic (StringyEval)
+		$v = eval "\\" . $pkgvar;
 	    };
 	    if (!defined $v) {
 		print {$DUMPTRACE_FH} "Devel::DumpTrace: ", 
@@ -971,7 +969,7 @@ sub evaluate {
 	} elsif (defined($PAD_OUR->{$sigvar})) {
 	    $v = $PAD_OUR->{$sigvar};
 	} else {
-	    $v = eval "\\$pkgvar";                    ## no critic (StringyEval)
+	    $v = eval "\\$pkgvar";
 	}
 	return '(' . hash_repr($v) . ')';
     }
@@ -987,7 +985,7 @@ sub evaluate {
 	    } elsif (defined($PAD_OUR->{$sigvar})) {
 		$v = $PAD_OUR->{$sigvar};
 	    } else {
-		eval { $v = eval "\\$pkgvar" };    ## no critic (StringyEval)
+		eval { $v = eval "\\$pkgvar" };
 		if (!defined $v) {
 		    print {$DUMPTRACE_FH} "Devel::DumpTrace: ",
 		        "Couldn't find $sigvar/$pkgvar in any appropriate scope.\n";
@@ -997,6 +995,7 @@ sub evaluate {
 	    return '(' . array_repr($v, @keys) . ')[';
 	} elsif ($varname =~ /^\d+$/) {
 	    # special regex match var $1,$2,...
+            # they were loaded into @matches in save_previous_regex_matches()
 	    $v = $matches[$varname];
 	    return dump_scalar($v);
 	} else {
@@ -1012,7 +1011,7 @@ sub evaluate {
 	    } elsif (defined($PAD_OUR->{$sigvar})) {
 		$v = ${$PAD_OUR->{$sigvar}};
 	    } else {
-		$v = eval "$pkgvar";               ## no critic (StringyEval)
+		$v = eval "$pkgvar";
 	    }
 	    return dump_scalar($v);
 	}
@@ -1036,44 +1035,30 @@ sub save_previous_regex_matches {
     return;
 }
 
-sub __END {
-    $_GLOBAL_DESTRUCTION = 1;
-    no warnings 'redefine';
-    *DB::DB = sub { };
-    untie $TRACE;
-    handle_ALL_deferred_output();
-}
-
 # RT#76864
 {
     # Devel::DumpTrace is typically loaded before any other module
-    # (from the -d:DumpTrace switch). Running this  threads-specific
+    # (from the -d:DumpTrace switch). Running this  thread-specific
     # code in a  CHECK  block gives the traced program a chance to
     # load  threads  later.
-    # We'll also use it to declare a late END block that is more
-    # likely to get executed first at global destruction.
     no warnings 'void';
     CHECK {
         if ($INC{'threads.pm'}) {
             $_THREADS = 1;
-            # idea for hijacking threads::create
-            # from  Devel::GlobalDestruction
-            my $orig_create = threads->can('create');
-            no warnings 'redefine';
-            *threads::create = sub {
-                { local $@; eval 'END { &__END }' };
-                goto $orig_create;
-            };
-            if ($INC{'threads/shared.pm'}) {
-                our $LOCKOBJ = 1;
-                threads::shared::share(\$LOCKOBJ);
-            } else {
-                our $LOCK_FH = File::Temp->new;
-                our $LOCKFILE = $LOCK_FH->filename;
-            }
+            require threads::shared;
+            our $LOCKOBJ = 1;  # to synchronize access to output stream
+            threads::shared::share(\$LOCKOBJ);
         }
-    }
-    INIT { eval 'END { &__END }' }
+    };
+}
+
+sub __END {
+    no warnings 'redefine';
+    *DB::DB = sub { };
+    *__inGD = sub () { 1 };
+    untie $TRACE;
+    handle_ALL_deferred_output() unless $_THREADS && threads->tid();
+    1;
 }
 
 END { &__END; }
@@ -1138,7 +1123,7 @@ Devel::DumpTrace - Evaluate and print out each line before it is executed.
 
 =head1 VERSION
 
-0.25
+0.26
 
 =head1 SYNOPSIS
 
@@ -1708,9 +1693,8 @@ L<Tie::Trace|Tie::Trace> provides facilities to watch the values
 of specific variables, including stack trace information about
 where and how the variables values were changed.
 
-An idea for hijacking the C<threads::create> method to facilitate
-clean up of programs with threads came from the 
-L<Devel::GlobalDestruction> module.
+Ideas from the L<Devel::GlobalDestruction> module were used to
+manage output during the end game of the traced script.
 
 =head1 AUTHOR
 
