@@ -1,5 +1,4 @@
-package Devel::DumpTrace::PPI;          ## no critic (FilenameMatchesPackage)
-
+package Devel::DumpTrace::PPI;
 use Devel::DumpTrace;
 use Devel::DumpTrace::Const;
 use PadWalker;
@@ -32,7 +31,7 @@ eval {use PPI;
 
 
 
-$Devel::DumpTrace::PPI::VERSION = '0.25';
+$Devel::DumpTrace::PPI::VERSION = '0.26';
 use constant ADD_IMPLICIT_ => 1;
 use constant DECORATE_FOR => 1;
 use constant DECORATE_FOREACH => 1;
@@ -67,7 +66,7 @@ sub Devel::DumpTrace::PPI::import {
     goto &Devel::DumpTrace::import;
 }
 
-# Overrides &get_source in Devel/DumpTrace.pm
+# Overrides  get_source  in Devel/DumpTrace.pm
 sub get_source_PPI {
     my ($file, $line) = @_;
 
@@ -84,7 +83,7 @@ sub _update_ppi_src_for_file {
     if ($file eq '-e'                       # code from  perl -e '...'
 	|| $file eq '-'                     # code from  cat prog.pl | perl
 	|| $file =~ /^\(eval \d+\)\[/) {    # code from an eval statement
-	no strict 'refs';                     ## no critic (NoStrict)
+	no strict 'refs';                   ## no critic (NoStrict)
 	my $all_code = join "", @{"::_<$file"}[1 .. $#{"::_<$file"}];
 	$doc = $ppi_doc{$file} = PPI::Document->new(\$all_code);
     } else {
@@ -461,7 +460,7 @@ sub handle_deferred_output_PPI {
     my $deferred = pop @{$Devel::DumpTrace::DEFERRED{"$sub : $file"}};
     return unless defined($deferred);
 
-    my @e = @{$deferred->{EXPRESSION}};
+    my @e = grep defined, @{$deferred->{EXPRESSION}};
     my $undeferred_output = join '', @e;
     my $deferred_pkg = $deferred->{PACKAGE};
     $Devel::DumpTrace::PAD_MY = $deferred->{MY_PAD};
@@ -1439,8 +1438,10 @@ sub __new_null_statement {
     return $NULL_STATEMENT->clone();
 }
 
-sub main::TEST {
-    # invoke with  perl -MDevel::DumpTrace::PPI -e TEST file
+sub main::TOKENIZE {
+    # invoke with  perl -MDevel::DumpTrace::PPI -e TOKENIZE file
+    # to see how an input script is tokenized by PPI and broken
+    # into statements by this package
     open(my $fh, '<', $ARGV[0])  or die;
     my $all_code = join '', <$fh>;
     close $fh;
@@ -1471,7 +1472,7 @@ Devel::DumpTrace::PPI - PPI-based version of Devel::DumpTrace
 
 =head1 VERSION
 
-0.25
+0.26
 
 =head1 SYNOPSIS
 
